@@ -1394,29 +1394,28 @@ static size_t flo_read(
     PyObject* bytes = PyObject_CallMethod(
         (PyObject*) user_data, "read", "n", (Py_ssize_t) size);
 
-    PyGILState_Release(gil_state);
-
-    if (bytes != NULL)
-    {
-      Py_ssize_t len;
-      char* buffer;
-
-      int result = PyBytes_AsStringAndSize(bytes, &buffer, &len);
-
-      if (result == -1 || (size_t) len < size)
-      {
-        Py_DECREF(bytes);
-        return i;
-      }
-
-      memcpy((char*) ptr + i * size, buffer, size);
-
-      Py_DECREF(bytes);
-    }
-    else
+    if (bytes == NULL) 
     {
+      PyGILState_Release(gil_state);
       return i;
     }
+
+    Py_ssize_t len;
+    char* buffer;
+
+    int result = PyBytes_AsStringAndSize(bytes, &buffer, &len);
+
+    if (result == -1 || (size_t) len < size)
+    {
+      Py_DECREF(bytes);
+      PyGILState_Release(gil_state);
+      return i;
+    }
+
+    memcpy((char*) ptr + i * size, buffer, size);
+
+    Py_DECREF(bytes);
+    PyGILState_Release(gil_state);
   }
 
   return count;
@@ -1444,12 +1443,11 @@ static size_t flo_write(
         (PyObject*) user_data, "write", "s#", (char*) ptr + i * size, size);
     #endif
 
+    Py_XDECREF(result);
     PyGILState_Release(gil_state);
 
     if (result == NULL)
       return i;
-
-    Py_DECREF(result);
   }
 
   return count;
@@ -1886,6 +1884,7 @@ static void StringMatchInstance_dealloc(
 
   Py_DECREF(object->offset);
   Py_DECREF(object->matched_data);
+  Py_DECREF(object->matched_length);
   Py_DECREF(object->xor_key);
 
   PyObject_Del(self);
@@ -1925,7 +1924,10 @@ static PyObject* StringMatchInstance_plaintext(
   StringMatchInstance* instance = (StringMatchInstance*) self;
   uint64_t xor_key = PyLong_AsUnsignedLongLong(instance->xor_key);
   if (xor_key == 0)
+  {
+      Py_INCREF(instance->matched_data);
       return instance->matched_data;
+  }
 
   int result = PyBytes_AsStringAndSize(instance->matched_data, &pb, &length);
   if (result == -1)
@@ -2626,7 +2628,7 @@ static PyObject* yara_compile(
 {
   static char *kwlist[] = {
     "filepath", "source", "file", "filepaths", "sources",
-    "includes", "externals", "error_on_warning", "include_callback", NULL};
+    "includes", "externals", "error_on_warning", "strict_escape", "include_callback", NULL};
 
   YR_COMPILER* compiler;
   YR_RULES* yara_rules;
@@ -2643,6 +2645,7 @@ static PyObject* yara_compile(
   PyObject* includes = NULL;
   PyObject* externals = NULL;
   PyObject* error_on_warning = NULL;
+  PyObject* strict_escape = NULL;
   PyObject* include_callback = NULL;
 
   Py_ssize_t pos = 0;
@@ -2659,7 +2662,7 @@ static PyObject* yara_compile(
   if (PyArg_ParseTupleAndKeywords(
         args,
         keywords,
-        "|ssOOOOOOO",
+        "|ssOOOOOOOO",
         kwlist,
         &filepath,
         &source,
@@ -2669,6 +2672,7 @@ static PyObject* yara_compile(
         &includes,
         &externals,
         &error_on_warning,
+        &strict_escape,
         &include_callback))
   {
     char num_args = 0;
@@ -2718,6 +2722,21 @@ static PyObject* yara_compile(
       }
     }
 
+    if (strict_escape != NULL)
+    {
+      if (PyBool_Check(strict_escape))
+      {
+        compiler->strict_escape = PyObject_IsTrue(strict_escape);
+      }
+      else
+      {
+        yr_compiler_destroy(compiler);
+        return PyErr_Format(
+            PyExc_TypeError,
+            "'strict_escape' param must be of boolean type");
+      }
+    }
+
     if (includes != NULL)
     {
       if (PyBool_Check(includes))
